#include "area.h"

#include <cmath>
#include <algorithm>

bool Area::isCollidingWithCircle(const Area& a) const
{
     int xDistance = a.xMidpoint() - xMidpoint();
     int yDistance = a.yMidpoint() - yMidpoint();
     double distance = std::sqrt(xDistance * xDistance + yDistance * yDistance);

     if(distance > (a.radius + radius))
          return false;
     else
          return true;
}

bool Area::isCollidingWithRect(const Area& a) const
{
     if(x() + w() < a.x()        ) return false;
     if(x()       > a.x() + a.w()) return false;
     if(y() + h() < a.y()        ) return false;
     if(y()       > a.y() + a.h()) return false;
     return true;
}

Area::Area(int nx, int ny, int nw, int nh)
{
     w(nw);
     h(nh);
     x(nx);       
     y(ny);
}

Area::Area()
{

}

void Area::x(int x)
{
     rect.x = x;
}

void Area::y(int y)
{
     rect.y = y;
}

void Area::xMidpoint(int x)
{
     rect.x = x - rect.w/2;
}

void Area::yMidpoint(int y)
{
     rect.y = y - rect.h/2;
}

void Area::w(int w)
{
     rect.w = w;
     radius = std::min(rect.w, rect.h) / 2;
}

void Area::h(int h)
{
     rect.h = h;
     radius = std::min(rect.w, rect.h) / 2;
}

int Area::x() const
{
     return rect.x;
}

int Area::y() const
{
     return rect.y;
}

int Area::xMidpoint() const
{
     return rect.x + rect.w/2;
}

int Area::yMidpoint() const
{
     return rect.y + rect.h/2;
}

int Area::w() const
{
     return rect.w;
}

int Area::h() const
{
     return rect.h;
}
