#include "gamescreen.h"

#include <iostream>
#include <algorithm>

GameScreen::GameScreen()
{
     std::cout << "GameScreen Created" << std::endl;
}

GameScreen::~GameScreen()
{
     std::cout << "GameScreen Destroyed" << std::endl;
     std::vector<Image*>::iterator it;
     
     delete player;
     
     for (it=baddies.begin(); it!=baddies.end(); it++)
     delete *it;
     baddies.clear();
     
     for (it=goldCoins.begin(); it!=goldCoins.end(); it++)
     delete *it;
     goldCoins.clear();
}

Screen::Result GameScreen::runScreen()
{
     status = None;
     SDL_Surface* screen = SDL_GetVideoSurface();
/*T*/const int ticksPerFrame = 1000/60;
     
     GameScreen::initialize();
     while(status == None)
     {
/*T*/     Uint32 ticksAtLoopStart = SDL_GetTicks();
          
          GameScreen::handleInput();
          GameScreen::updateAll();
          GameScreen::drawAll(screen);
          
/*T*/     int time_to_wait = ticksPerFrame - (SDL_GetTicks() - ticksAtLoopStart);
/*T*/     if(time_to_wait > 0)
/*T*/          SDL_Delay(time_to_wait);
     }
     
     return status;
}

void GameScreen::initialize()
{
     player = new CirclingImage("redball.bmp", 320, 240);
     player->setCircleParameters(40, .09);
     
     baddies.push_back(new Image("greenball.bmp",  0,  0));
     baddies.push_back(new Image("greenball.bmp",100,  0));
     baddies.push_back(new Image("greenball.bmp",100,100));
     baddies.push_back(new Image("greenball.bmp",  0,100));
     
     goldCoins.push_back(new Image("gold.bmp",  50,   0));
     goldCoins.push_back(new Image("gold.bmp", 150,   0));
     goldCoins.push_back(new Image("gold.bmp",   0,  50));
     goldCoins.push_back(new Image("gold.bmp",   0, 150));
}

void GameScreen::handleInput()
{
     SDL_Event event;
     while(SDL_PollEvent(&event))
     {
          switch (event.type)
          {
          case SDL_MOUSEBUTTONDOWN:
               break;
          case SDL_MOUSEMOTION:
               player->setCircleCenter(event.motion.x, event.motion.y);
               break;
          case SDL_QUIT:
               status = Exit;
               break;
          }
     }
}

void GameScreen::updateAll()
{
     std::vector<Image*>::iterator it;
     player->update();
     for (it=baddies.begin(); it!=baddies.end(); it++)
     {
          (*it)->update();
          if((*it)->isCollidingWithCircle(*player))
          status = Defeat; //Game over, player died
     }
     for (it=goldCoins.begin(); it!=goldCoins.end(); )
     {
          (*it)->update();
          if((*it)->isCollidingWithCircle(*player))
          {
               std::cout << "You picked up a gold coin!" << std::endl;
               //TODO: Fix Memory Leak
               it = goldCoins.erase(it);
          }else{
               it++;
          }
     }
     if (goldCoins.size() == 0)
     status = Victory; //Game over, player got all the gold
}

void GameScreen::drawAll(SDL_Surface* screen)
{
     std::vector<Image*>::iterator it;
     SDL_FillRect(screen, NULL, 0x000000);
     for (it=baddies.begin(); it!=baddies.end(); it++)
     (*it)->draw(screen);
     for (it=goldCoins.begin(); it!=goldCoins.end(); it++)
     (*it)->draw(screen);
     player->draw(screen);
     SDL_Flip(screen);
}
